@@ -21,10 +21,23 @@
 #ifndef HEADER_APP_GLOM
 #define HEADER_APP_GLOM
 
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
+//TODO: Remove this when maemomm's gtkmm has been updated. 7th September 2009.
+#ifdef GLOM_ENABLE_MAEMO
+//Because earlier versions of gtkmm/enums.h did not include this, so  
+//GTKMM_MAEMO_EXTENSIONS_ENABLED was not defined, 
+//leading to a compiler error in hildonmm/button.h
+#include <gtkmmconfig.h>
+#include <gtkmm/enums.h>
+
+#include <hildonmm/app-menu.h>
+#include <glom/navigation/maemo/pickerbutton_table.h>
+#endif //GLOM_ENABLE_MAEMO
+
 #include <glom/bakery/app_withdoc_gtk.h>
 #include <glom/frame_glom.h>
 
-#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
 
 //Avoid including the header here:
@@ -78,8 +91,10 @@ public:
   void fill_menu_reports(const Glib::ustring& table_name);
   void fill_menu_print_layouts(const Glib::ustring& table_name);
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   void do_menu_developer_fields(Gtk::Window& parent, const Glib::ustring table_name);
   void do_menu_developer_relationships(Gtk::Window& parent, const Glib::ustring table_name);
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
   ///Whether to show the generated SQL queries on stdout, for debugging.
   bool get_show_sql_debug() const;
@@ -94,17 +109,28 @@ protected:
 
 private:
   virtual void init_layout(); //override.
-  virtual void init_menus_file(); //override.
   virtual void init_menus(); //override.
-  virtual void init_menus_help(); //override
   virtual void init_toolbars(); //override
   virtual void init_create_document(); //override
   virtual bool on_document_load(); //override.
   virtual void on_document_close(); //override.
+  virtual void update_window_title(); //override.
+
+#ifndef GLOM_ENABLE_MAEMO
+  virtual void init_menus_file(); //override.
+  virtual void init_menus_help(); //override
+#endif //GLOM_ENABLE_MAEMO
 
   bool offer_new_or_existing();
 
+#ifndef GLOM_ENABLE_MAEMO
   void on_menu_help_contents();
+#endif //GLOM_ENABLE_MAEMO
+
+  /** Check that the file's hosting mode is supported by this build and 
+   * tell the user if necessary.
+   */
+  bool check_document_hosting_mode_is_supported(Document* document);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   void existing_or_new_new();
@@ -142,8 +168,6 @@ private:
   Glib::ustring ui_file_select_open_with_browse(bool& browsed, EpcServiceInfo*& browsed_server, Glib::ustring& browsed_service_name, const Glib::ustring& starting_folder_uri = Glib::ustring());
 #endif // !G_OS_WIN32
 
-  virtual void on_menu_file_open(); //overridden.
-  virtual void on_menu_file_close(); //override.
   virtual void document_history_add(const Glib::ustring& file_uri); //overridden.
 
   virtual void new_instance(const Glib::ustring& uri = Glib::ustring()); //Override
@@ -171,6 +195,13 @@ private:
   Glib::RefPtr<Gtk::ToggleAction> m_action_show_layout_toolbar;
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+#ifdef GLOM_ENABLE_MAEMO
+  Hildon::AppMenu m_maemo_appmenu;
+  PickerButton_Table m_appmenu_button_table;
+
+  void on_appmenu_button_table_value_changed();
+#endif //GLOM_ENABLE_MAEMO
+
   Glib::RefPtr<Gtk::ToggleAction> m_toggleaction_network_shared;
   sigc::connection m_connection_toggleaction_network_shared;
 
