@@ -64,6 +64,9 @@
 
 #ifdef GLOM_ENABLE_MAEMO
 #include <hildonmm/note.h>
+#include <hildonmm/entry.h>
+#include <hildonmm/text-view.h>
+#include <hildonmm/button.h>
 #endif
 
 #include <glom/filechooser_export.h>
@@ -136,7 +139,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
   #ifndef GLOM_ENABLE_MAEMO
   m_pEntry_QuickFind = Gtk::manage(new Gtk::Entry());
   #else
-  m_pEntry_QuickFind = Gtk::manage(new Hildon::Entry());
+  m_pEntry_QuickFind = Gtk::manage(new Hildon::Entry(Gtk::Hildon::SIZE_AUTO));
   #endif
   m_pEntry_QuickFind->signal_activate().connect(
    sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) ); //Pressing Enter here is like pressing Find.
@@ -146,7 +149,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
   m_pButton_QuickFind = Gtk::manage(new Gtk::Button(_("_Find"), true));
   #else
   m_pButton_QuickFind = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_AUTO, 
-    Hildon::BUTTON_ARRANGEMENT_VERTICAL, _("Find")));
+    Hildon::BUTTON_ARRANGEMENT_VERTICAL, _("Find"), _("Search for records")));
   #endif
   m_pButton_QuickFind->signal_clicked().connect(
     sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) );
@@ -820,6 +823,28 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
 
             const Glib::ustring field_text = field->to_file_format(value);
 
+            if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
+            {
+              // Some extra debug checks, 
+              // though we believe that all these problems are now fixed in File::to_file_format():
+
+              const char* newline_to_find = "\r\n";
+              size_t pos = field_text.find_first_of(newline_to_find);
+              if(pos != std::string::npos)
+              {
+                std::cerr << "export: binary data field text contains an unexpected newline: " << field_text << std::endl;
+                continue;
+              }
+
+              const char* quote_to_find = "\"";
+              pos = field_text.find_first_of(quote_to_find);
+              if(pos != std::string::npos)
+              {
+                std::cerr << "export: binary data field text contains an unexpected quote: " << field_text << std::endl;
+                continue;
+              }
+            }
+            
             if(layout_item->get_glom_type() == Field::TYPE_TEXT)
             {
               //The CSV RFC says text may be quoted and should be if it has newlines:
@@ -828,11 +853,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
             else
               row_string += field_text;
 
-            if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
-            {
-              if(!Conversions::value_is_empty(value))
-                std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
-            }
+           
             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
           //}
         }
