@@ -19,9 +19,9 @@
  */
 
 #include "field.h"
-#include <glom/libglom/connectionpool.h>
+#include <libglom/connectionpool.h>
 #include "glomconversions.h"
-#include <glom/libglom/utils.h>
+#include <libglom/utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -110,7 +110,16 @@ Field::glom_field_type Field::get_glom_type() const
 
 void Field::set_glom_type(glom_field_type fieldtype)
 {
+  glom_field_type old_type = m_glom_type;
+
+  // Set glom type from fieldinfo if it represents a different type.
   m_glom_type = fieldtype;
+
+  // Reset default value if the default value type does not match the new
+  // glom type.
+  // TODO: Should we try to convert the default value to the new type here?
+  if(fieldtype != old_type)
+    set_default_value(Gnome::Gda::Value()); //TODO: Try to convert it, maybe by rendering and parsing it.
 }
 
 Glib::RefPtr<Gnome::Gda::Column> Field::get_field_info()
@@ -127,7 +136,6 @@ void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
 {
   m_field_info = fieldinfo;
 
-  // Set glom type from fieldinfo if it represents a different type.
   // Also take fallback types into account as fieldinfo might originate from
   // the database system directly.
   GType new_type = fieldinfo->get_g_type();
@@ -152,6 +160,33 @@ void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
 
   if(cur_type == G_TYPE_NONE)
     set_glom_type( get_glom_type_for_gda_type(fieldinfo->get_g_type()) );
+
+  Gnome::Gda::Value value = get_default_value();
+  if(!value.is_null())
+  {
+    // Check that the default value is consistent with the field's type
+    // TODO: Basically copied from set_default_value(). Maybe this check should
+    // be moved into an extra function.
+    GType cur_type = get_gda_type_for_glom_type(get_glom_type());
+    const FieldTypes* pFieldTypes = NULL;
+
+    ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
+    if(pConnectionPool)
+      pFieldTypes = pConnectionPool->get_field_types();
+
+    // Take into account that value might be one of the fallback types
+    if(pFieldTypes)
+    {
+      while(cur_type != value.get_value_type() && cur_type != G_TYPE_NONE)
+        cur_type = pFieldTypes->get_fallback_type_for_gdavaluetype(cur_type);
+    }
+
+    if(!value.is_null() && value.get_value_type() != cur_type)
+    {
+      g_warning("Field::set_field_info: New field's default value type (%s) does not match field type (%s). Resetting default value.", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
+      m_field_info->set_default_value(Gnome::Gda::Value());
+    }
+  }
 }
 
 sharedptr<Relationship> Field::get_lookup_relationship() const
@@ -180,393 +215,157 @@ bool Field::get_is_lookup() const
   return ( m_lookup_relationship && (!m_strLookupField.empty()) );
 }
 
-namespace { //anonymous
-
-//A copy of PQescapeString from the Postgres source, to avoid linking with libpql directly, 
-//and to use until we can use the latest libgda, which has an equivalent.
-//TODO: Now that we use libgda-3.0, actually use that equivalent.
-
-#define SQL_STR_DOUBLE(ch) ((ch) == '\'' || (ch) == '\\')
-
-/*
- * Escaping arbitrary strings to get valid SQL literal strings.
- *
- * Replaces "\\" with "\\\\" and "'" with "''".
- *
- * length is the length of the source string.  (Note: if a terminating NUL
- * is encountered sooner, PQescapeString stops short of "length"; the behavior
- * is thus rather like strncpy.)
- *
- * For safety the buffer at "to" must be at least 2*length + 1 bytes long.
- * A terminating NUL character is added to the output string, whether the
- * input is NUL-terminated or not.
- *
- * Returns the actual length of the output (not counting the terminating NUL).
- */
-size_t
-Glom_PQescapeString(char *to, const char *from, size_t length)
-{
-	const char *source = from;
-	char	   *target = to;
-	size_t		remaining = length;
-
-	while (remaining > 0 && *source != '\0')
-	{
-		if(SQL_STR_DOUBLE(*source))
-			*target++ = *source;
-
-		*target++ = *source++;
-		remaining--;
-	}
-
-	/* Write the terminating NUL character. */
-	*target = '\0';
-
-	return target - to;
-}
-
-} //anonymous
-
-/// Escape text, including text that is the result of get_escaped_binary_data().
-static std::string glom_escape_text(const std::string& src)
-{
-  if(src.empty())
-    return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-  else
-  {
-    const size_t len = src.size();
-    char* to = (char*)malloc(sizeof(char) * 2 * (len + 1)); //recommended size for to.
-    const size_t len_escaped = Glom_PQescapeString(to, src.c_str(), len);
-    if(!len_escaped)
-    {
-      std::cerr << "glom_escape_text(): Glom_PQescapeString() failed with text: " << src << std::endl;
-
-      if(to)
-        free(to);
-
-      return "''";
-    }
-    else
-    {
-      std::string escaped(to, len_escaped);
-      free(to);
-
-      //Also escape any ";" characters, because these also cause problems, at least with libgda:
-      //See bug #326325.
-      escaped = Utils::string_replace(escaped, ";", "\\073");
-
-      return ("'" + escaped + "'"); //Add single-quotes. Actually escape it 
-      //std::cout << "glom_escape_text: escaped and quoted: " << str << std::endl;
-    }
-  }
-}
-
-#define ISFIRSTOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '3')
-#define ISOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '7')
-#define OCTVAL(CH) ((CH) - '0')
-
-/// Unescape text that was escaped by the above function. Only unescapes
-/// quoted strings. Nonquoted strings are returned without being modified.
-static std::string glom_unescape_text(const std::string& str)
-{
-  std::string::const_iterator iter = str.begin();
-  if(iter == str.end()) return str;  // Empty string
-  if(*iter != '\'') return str; // Non-quoted
-  ++ iter;
-
-  std::string result;
-
-  while(iter != str.end())
-  {
-    // End here if this is the terminating quotation character, or unescape
-    // '' to '.
-    if(*iter == '\'')
-    {
-      ++ iter;
-      if(iter == str.end() || *iter != '\'') break;
-      result += '\'';
-      ++ iter;
-    }
-    /* double quotes are not escaped, so don't unescape them. */
-#if 0
-    // Unescape "" to ".
-    else if(*iter == '\"')
-    {
-      ++ iter;
-      if(iter == str.end()) break;
-      result += '\"';
-      ++ iter;
-    }
-#endif
-    // Escape sequence beginning with backslash.
-    else if(*iter == '\\')
-    {
-      ++ iter;
-      if(iter == str.end()) break;
-
-      // Escaped backslash
-      if(*iter == '\\')
-      {
-        result += '\\';
-        ++ iter;
-      }
-      // Some octal representation
-      else if(ISFIRSTOCTDIGIT(*iter))
-      {
-        unsigned char byte = OCTVAL(*iter);
-
-        ++ iter;
-        if(iter != str.end() && ISOCTDIGIT(*iter))
-        {
-          byte = (byte << 3) | OCTVAL(*iter);
-          ++ iter;
-          if(iter != str.end() && ISOCTDIGIT(*iter))
-          {
-            byte = (byte << 3) | OCTVAL(*iter);
-            ++ iter;
-          }
-        }
-
-        result += byte;
-      }
-    }
-    else
-    {
-      // Take char as is
-      result += *iter;
-      ++ iter;
-    }
-  }
-
-  return result;
-}
-
-Glib::ustring Field::sql(const Gnome::Gda::Value& value, sql_format format) const
+Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const
 {
   //g_warning("Field::sql: glom_type=%d", get_glom_type());
 
-  if(value.is_null())
+  if(value.is_null() && (get_glom_type() == TYPE_TEXT))
   {
-    switch(get_glom_type())
-    {
-      case(TYPE_TEXT):
-      {
-        return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-        break;
-      }
-      case(TYPE_DATE):
-      case(TYPE_TIME):
-      case(TYPE_NUMERIC):
-      case(TYPE_IMAGE):
-      {
-        return "NULL";
-        break;
-      }
-      case(TYPE_INVALID):
-      {
-        g_warning("Field::sql(): The field type is INVALID.");
-        return "NULL";
-        break;
-      }
-      default:
-      {
-        //Don't deal with these here.
-        break;
-      }
-    }
+    return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
   }
 
-  Glib::ustring str;
 
-  switch(get_glom_type())
+  //Use libgda's DataHandler to get the string for SQL:
+  Glib::RefPtr<const Gnome::Gda::ServerProvider> provider = connection->get_provider();
+  if(!provider)
   {
-    case(TYPE_TEXT):
-    {
-      if(value.is_null())
-        return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-      else
-      {
-        str = value.get_string();
-        str = glom_escape_text(str);
-      }
+    std::cerr << "Field::sql(): The ServerProvider was null." << std::endl;
+    return Glib::ustring();
+  } 
 
-      break;
-    }
-    case(TYPE_DATE):
-    case(TYPE_TIME):
-    {
-      NumericFormat format_ignored; //Because we use ISO format.
-      str = Conversions::get_text_for_gda_value(m_glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
-      if(str != "NULL")
-         str = "'" + str + "'"; //Add single-quotes.
-
-      break;
-    }
-    case(TYPE_NUMERIC):
-    {
-      str =  Conversions::get_text_for_gda_value(m_glom_type, value, std::locale() /* SQL uses the C locale */); //No quotes for numbers.
-      break;
-    }
-    case(TYPE_BOOLEAN):
-    {
-      if(G_VALUE_TYPE(value.gobj()) == G_TYPE_BOOLEAN)
-        str = (value.get_boolean() ? "TRUE" : "FALSE" );
-      else
-        str = "FALSE";
-
-      break;
-    }
-    case(TYPE_IMAGE):
-    {
-      if(value.get_value_type() == GDA_TYPE_BINARY)
-      {
-        long buffer_length;
-        const guchar* buffer = value.get_binary(buffer_length);
-        if(buffer && buffer_length > 0)
-        {
-          if(format == SQL_FORMAT_POSTGRES)
-          {
-            //Get the escaped text that represents the binary data:
-            const std::string escaped_binary_data = Conversions::escape_binary_data_postgres((guint8*)buffer, buffer_length);
-            //Now escape that text (to convert \ to \\, for instance):
-            //The E prefix indicates ""escape" string constants, which are an extension to the SQL standard"
-            //Otherwise, we get a warning when using large escaped strings:
-            if(format == SQL_FORMAT_POSTGRES)
-              str = "E" + glom_escape_text(escaped_binary_data) /* has quotes */ + "::bytea";
-          }
-          else
-          {
-            const std::string escaped_binary_data = Conversions::escape_binary_data_sqlite((guint8*)buffer, buffer_length);
-            str = "x'" + escaped_binary_data + "'";
-          }
-        }
-      }
-      else
-      {
-        g_warning("Field::sql(): glom_type is TYPE_IMAGE but gda type is not VALUE_TYPE_BINARY");
-      }
-
-      break;
-    }
-    default:
-    {
-      str = value.to_string();
-      if(str.empty() && (m_glom_type != Field::TYPE_TEXT))
-        str = "NULL"; //This has probably been handled in get_text_for_gda_value() anyway.
-
-      break;
-    }
+  const GType gda_type = get_gda_type_for_glom_type(m_glom_type);
+  Glib::RefPtr<const Gnome::Gda::DataHandler> datahandler = 
+    provider->get_data_handler_g_type(connection, gda_type);
+  if(datahandler)
+  {
+    //Note that this does seems to add the needed quotes too, for instance around strings,
+    //though that is not clearly documented. See bug http://bugzilla.gnome.org/show_bug.cgi?id=572220
+    return datahandler->get_sql_from_value(value);
+  }
+  else
+  {
+    std::cerr << "Field::sql(): The DataHandler was null." << std::endl;
+    return Glib::ustring();
   }
 
-  return str;
+  return Glib::ustring();
 }
 
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
-  sql_format format = ConnectionPool::get_instance()->get_sql_format();
-  return sql(value, format);
+  sharedptr<SharedConnection> connection;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+    connection = ConnectionPool::get_instance()->connect();
+  }
+  catch (Glom::ExceptionConnection& ex)
+  {
+    //TODO: Do something more useful here
+    std::cerr << "Field::sql, connection failed: " << ex.what() << std::endl;
+  }
+#else
+  std::auto_ptr<Glom::ExceptionConnection> ex;
+  connection = ConnectionPool::get_instance()->connect(ex);
+  if (ex.get())
+    std::cerr << "Field::sql, connection failed: " << ex->what() << std::endl;
+#endif
+  if(connection)
+  {
+    Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connection->get_gda_connection();
+    if(gda_connection)
+      return sql(value, gda_connection);
+  }
+
+  return Glib::ustring();
 }
 
-Gnome::Gda::Value Field::from_sql(const Glib::ustring& str, sql_format format, bool& success) const
+#define GLOM_QUOTE_FOR_FILE_FORMAT "\""
+
+Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value) const
+{
+  return to_file_format(value, m_glom_type);
+}
+
+Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value, glom_field_type glom_type)
+{
+  if(glom_type == TYPE_IMAGE)
+  {
+    if(!value.gobj() || (value.get_value_type() != GDA_TYPE_BINARY))
+      return Glib::ustring();
+ 
+    const GdaBinary* gdabinary = gda_value_get_binary(value.gobj());
+    if(!gdabinary)
+      return Glib::ustring();
+    else
+    {
+      gchar* str = gda_binary_to_string(gdabinary, 0);
+      Glib::ustring result = (str) ? 
+        Glib::ustring(Glib::ScopedPtr<char>(str).get()) : Glib::ustring();
+
+      //Avoid arbitrary newlines in this text.
+      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+      result = Utils::string_replace(result, "\n", "\\012");
+
+      //Avoid arbitrary newlines in this text.
+      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+      result = Utils::string_replace(result, "\r", "\\015");
+
+      //Escape any quotes in this text:
+      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+      return Utils::string_replace(result, "\"", "\\042");
+    }
+  }
+  
+  NumericFormat format_ignored; //Because we use ISO format.
+  const Glib::ustring result = Conversions::get_text_for_gda_value(glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
+  
+  //Escape " as "", as specified by the CSV RFC:
+  return Utils::string_replace(result, GLOM_QUOTE_FOR_FILE_FORMAT, GLOM_QUOTE_FOR_FILE_FORMAT GLOM_QUOTE_FOR_FILE_FORMAT);
+}
+
+namespace
+{
+  // Changes the type of a given value
+  void value_reinit(GValue* value, GType g_type)
+  {
+    if(G_IS_VALUE(value) && G_VALUE_TYPE(value) != g_type)
+      g_value_unset(value);
+
+    if(!G_IS_VALUE(value))
+      g_value_init(value, g_type);
+  }
+}
+
+Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, bool& success) const
+{
+  return from_file_format(str, m_glom_type, success);
+}
+
+Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, glom_field_type glom_type, bool& success)
 {
   success = true;
-  switch(m_glom_type)
+  
+  if(glom_type == TYPE_IMAGE)
   {
-  case TYPE_TEXT:
+    if(str.empty())
+      return  Gnome::Gda::Value();
+
+    GdaBinary* gdabinary = gda_string_to_binary(str.c_str());
+    if(!success || !gdabinary)
+      return Gnome::Gda::Value();
+    else
     {
-      return Gnome::Gda::Value(glom_unescape_text(str));
+      Gnome::Gda::Value value;
+      value_reinit(value.gobj(), GDA_TYPE_BINARY);
+      gda_value_take_binary(value.gobj(), gdabinary);
+      return value; 
     }
-  case TYPE_DATE:
-  case TYPE_TIME:
-    {
-      if(str == "NULL") return Gnome::Gda::Value();
-      Glib::ustring unescaped = glom_unescape_text(str);
-
-      NumericFormat format_ignored; //Because we use ISO format.
-      return Conversions::parse_value(m_glom_type, unescaped, format_ignored, success, true);
-    }
-  case TYPE_NUMERIC:
-    {
-      //No quotes for numbers.
-      NumericFormat format_ignored; //Because we use ISO format.
-      return Conversions::parse_value(m_glom_type, str, format_ignored, success, true);
-    }
-  case TYPE_BOOLEAN:
-    {
-      if(str.lowercase() == "true")
-        return Gnome::Gda::Value(true);
-      return Gnome::Gda::Value(false);
-    }
-  case TYPE_IMAGE:
-    {
-      if(str == "NULL") return Gnome::Gda::Value();
-
-      // We store the data into the format E'some escaped data'::bytea, and we
-      // now expect it in exactly that format now.
-      // We operate on the raw std::string since access into the Glib::ustring
-      // is expensive, and we only check for ASCII stuff anyway.
-      const std::string& raw = str.raw();
-
-      Glib::ustring unescaped;
-      switch(format)
-      {
-      case SQL_FORMAT_POSTGRES:
-        if(raw.length() >= 10 &&
-           raw.compare(0, 2, "E'") == 0 && raw.compare(raw.length() - 8, 8, "'::bytea") == 0)
-        {
-          unescaped = glom_unescape_text(raw.substr(1, raw.length() - 8));
-	}
-	// The E can be omitted, and it is in some example files, such as
-	// example_smallbusiness.glom. We need to parse this to convert the
-	// data into sqlite format, when the small bussiness example is loaded
-	// into a sqlite database.
-	else if(raw.length() >= 9 &&
-	        raw.compare(0, 1, "'") == 0 && raw.compare(raw.length() - 8, 8, "'::bytea") == 0)
-	{
-          unescaped = glom_unescape_text(raw.substr(0, raw.length() - 7));
-        }
-
-        if(!unescaped.empty())
-        {
-          gsize length;
-          guint8* binary_data = Conversions::unescape_binary_data_postgres(unescaped, length);
-          if(binary_data)
-          {
-            Gnome::Gda::Value value;
-            value.set(binary_data, length);
-            g_free(binary_data);
-            return value;
-          }
-        }
-
-        success = false;
-        return Gnome::Gda::Value();
-      case SQL_FORMAT_SQLITE:
-        if(raw.length() >= 3 &&
-           (raw.compare(0, 2, "x'") == 0 || raw.compare(0, 2, "X'")) &&
-           raw.compare(raw.length() - 1, 1, "'") == 0)
-        {
-          gsize length;
-          guint8* binary_data = Conversions::unescape_binary_data_sqlite(raw.substr(2, raw.length()), length);
-          if(binary_data)
-          {
-            Gnome::Gda::Value value;
-            value.set(binary_data, length);
-            g_free(binary_data);
-            return value;
-          }
-        }
-
-        success = false;
-        return Gnome::Gda::Value();
-      default:
-        g_assert_not_reached();
-        break;
-      }
-    }
-  default:
-    g_assert_not_reached();
-    break;
+  }
+  else
+  {
+    NumericFormat format_ignored; //Because we use ISO format.
+    return Conversions::parse_value(glom_type, str, format_ignored, success, true);
   }
 }
 
@@ -581,7 +380,7 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
       if(value.is_null())
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
-        return ("'%" + value.to_string() + "%'"); //Add single-quotes. Actually escape it.
+        return ("'%" + value.to_string() + "%'"); //Add single-quotes. TODO: Actually escape it.
         
       break;
     }
@@ -603,7 +402,11 @@ Glib::ustring Field::sql_find_operator() const
   {
     case(TYPE_TEXT):
     {
-      return "ILIKE"; //"LIKE"; ILIKE is a postgres extension for locale-dependent case-insensitive matches.
+      ConnectionPool* connection_pool = ConnectionPool::get_instance();
+      if(connection_pool && connection_pool->get_backend())
+        return connection_pool->get_string_find_operator();
+      else
+        return "LIKE"; // Default
       break;
     }
     case(TYPE_DATE):
@@ -670,7 +473,27 @@ Gnome::Gda::Value Field::get_default_value() const
 
 void Field::set_default_value(const Gnome::Gda::Value& value)
 {
-  m_field_info->set_default_value(value);
+  // TODO: Allow setting a NULL default value when glom type is invalid?
+
+  // Verify that the value matches the type of the field.
+  GType cur_type = get_gda_type_for_glom_type(get_glom_type());
+  const FieldTypes* pFieldTypes = NULL;
+
+  ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
+  if(pConnectionPool)
+    pFieldTypes = pConnectionPool->get_field_types();
+
+  // Take into account that value might be one of the fallback types
+  if(pFieldTypes)
+  {
+    while(cur_type != value.get_value_type() && cur_type != G_TYPE_NONE)
+      cur_type = pFieldTypes->get_fallback_type_for_gdavaluetype(cur_type);
+  }
+
+  if(value.is_null() || value.get_value_type() == cur_type)
+    m_field_info->set_default_value(value);
+  else
+    g_warning("Field::set_default_value: Cannot set incompatible default value: Default value has type %s, but field has type %s", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
 }
 
 Glib::ustring Field::get_sql_type() const
@@ -716,6 +539,7 @@ Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& valu
 
   const GType gtype = value.get_value_type();
 
+  /*
   const GType field_type = m_field_info->get_g_type();
   if(m_field_info->get_g_type() != gtype)
   {
@@ -723,11 +547,19 @@ Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& valu
     // GdaBinary, but the field type is GdaBlob. This is not an optimal
     // solution this way. We should maybe check fallback types here, or
     // investigate why the field type is not GdaBinary as well.
-    //g_warning("Field::get_holder(): Field type (%s) and value type (%s) don't match!\n", g_type_name(field_type), g_type_name(gtype));
+    // Maybe get_gda_type_for_glom_type() should already return fallback
+    // types if necessary.
+    std::cout << "DEBUG: Field::get_holder(): Field type " << g_type_name(field_type) << " and value type " << g_type_name(gtype) << " don't match." << std::endl;
   }
+  */
 
   Glib::RefPtr<Gnome::Gda::Holder> holder = Gnome::Gda::Holder::create(gtype, real_name);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   holder->set_value_as_value(value);
+#else
+  std::auto_ptr<Glib::Error> ex;
+  holder->set_value_as_value(value, ex);
+#endif
   return holder;
 }
 
