@@ -21,9 +21,9 @@
 #ifndef GLOM_MODE_DATA_LAYOUT_WIDGET_BASE_H
 #define GLOM_MODE_DATA_LAYOUT_WIDGET_BASE_H
 
-#include <glom/libglom/data_structure/layout/layoutitem.h>
+#include <libglom/data_structure/layout/layoutitem.h>
 #include <gtkmm.h>
-#include "../mode_data/treestore_layout.h" //Forthe enum.
+#include <glom/mode_design/layout/treestore_layout.h> //Forthe enum.
 
 #include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
@@ -52,22 +52,19 @@ public:
     TYPE_NOTEBOOK,
     TYPE_PORTAL,
     TYPE_BUTTON,
-    TYPE_TEXT
+    TYPE_TEXT,
+    TYPE_IMAGE
   };
 
-  //Popup-menu:
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual void setup_menu();
-  virtual void on_menupopup_activate_layout();
-  virtual void on_menupopup_activate_layout_properties();
-  virtual void on_menupopup_add_item(enumType item);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   typedef sigc::signal<void> type_signal_layout_changed;
+
+  /// Signals that the layout has changed, so it should be saved to the document again.
   type_signal_layout_changed signal_layout_changed();
 
   typedef sigc::signal<void, enumType> type_signal_layout_item_added;
+
+  ///Requests the addition of an item:
   type_signal_layout_item_added signal_layout_item_added();
 
   //Allow a child widget to delegate to a parent widget:
@@ -81,27 +78,36 @@ public:
 
   virtual void set_read_only(bool read_only = true);
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  void set_dnd_in_progress(bool drag = true);
+  bool get_dnd_in_progress();
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+    
 protected:
   virtual App_Glom* get_application() const; // = 0;
 
+
+  static void apply_formatting(Gtk::Widget& widget, const FieldFormatting& formatting);
+
+protected: //TODO: Add accessor?
   sharedptr<LayoutItem> m_pLayoutItem;
+
+protected: //TODO: Add accessor?
   Glib::ustring m_table_name;
 
+private:
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+  /// Signals that the layout has changed, so it should be saved to the document again.
   type_signal_layout_changed m_signal_layout_changed;
+
+  /// Requests the addition of an item.
   type_signal_layout_item_added m_signal_layout_item_added;
 
   type_signal_user_requested_layout m_signal_user_requested_layout;
   type_signal_user_requested_layout_properties m_signal_user_requested_layout_properties;
 
-  Gtk::Menu* m_pMenuPopup;
-
-  //TODO_Performance: //Presumably we waste lots of memory by having this in each layout widget. Maybe we can use one shared menu.
-  Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
-  Glib::RefPtr<Gtk::UIManager> m_refUIManager;
-
-  Glib::RefPtr<Gtk::Action> m_refContextLayout, m_refContextLayoutProperties;
-  Glib::RefPtr<Gtk::Action> m_refContextAddField, m_refContextAddRelatedRecords, m_refContextAddGroup, m_refContextAddNotebook, m_refContextAddButton, m_refContextAddText;
+  bool m_drag_in_progress;
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 };
 
